<?php

declare(strict_types=1);

namespace Drupal\easy_carousel\Plugin\Block;

use Drupal\easy_carousel\Entity\Carousel;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a carousel base block.
 */
abstract class CarouselBaseBlock extends BlockBase implements ContainerFactoryPluginInterface {

  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Return the render mode id.
   *
   * The render mode is the responsible of render the selected carousel.
   *
   * @return string
   *   The plugin render mode id.
   */
  public function getRenderModeId(): string { return ''; }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'selected_carousel' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    return [
      'selected_carousel' => [
        '#type' => 'entity_autocomplete',
        '#title' => $this->t('Select the carousel to display'),
        '#required' => TRUE,
        '#target_type' => 'carousel',
        '#tags' => TRUE,
        '#default_value' => $this->getSelectedCarousel() ?? [],
        '#selection_handler' => 'default',
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['selected_carousel'] = $form_state->getValue('selected_carousel');
    $this->configuration['last_updated'] = time();
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    // You can build the default view array for the
    // entity in this way:
    // $viewBuilder = $this->entityTypeManager->getViewBuilder('carousel');
    // $carouselType = $this->configuration['carousel_type'];
    // $build = $viewBuilder->view($carousel, $carouselType);
    // $build['slides'] = [];
    // $itemBuilder = $this->entityTypeManager->getViewBuilder('carousel_item');
    // foreach($carousel->getItems() as $it) {
    //   $build['slides'][] = $itemBuilder->view($it, $carouselType);
    // }
    // $build['configuration'] = $this->configuration;
    // $build['#attached']['drupalSettings']['easy_carousel'] = [
    //   'config' => $this->configuration,
    // ];
    // $build['#attached']['library'] = ['easy_carousel/default'];

    $carousel = $this->getSelectedCarousel();
    if ($carousel) {
      $configuration = $this->configuration;
      $renderMode = $this->getRenderModeId();
      /** @var \Drupal\easy_carousel\Render\RenderModeManager $manager */
      $manager = \Drupal::service('plugin.manager.easy_carousel_render_mode');
      $plugin = $manager->createInstance($renderMode);
      $build = $plugin->render($carousel, $configuration);
      return $build;
    }
    else {
      $build['carousel'] = [
        '#markup' => $this->t('No carousel found.'),
      ];
    }
    return $build;
  }

  /**
   * Get the stored Carousel saved in the configuration.
   *
   * @return Carousel|null
   *   The carousel entity or null if not exists.
   */
  public function getSelectedCarousel():? Carousel {
    $id = $this->configuration['selected_carousel'][0]['target_id'] ?? NULL;
    if ($id) {
      $carousel = $this->entityTypeManager->getStorage('carousel')->load($id);
      return $carousel;
    }
    return NULL;
  }

}
