<?php

namespace Drupal\easy_carousel\Render;

use Drupal;
use Drupal\Component\Plugin\PluginBase;
use Drupal\easy_carousel\CarouselInterface;

/**
 * Default plugin implementation of the CarouselRenderInterface.
 *
 * Provides common methods for render modes.
 *
 * All plugins that implement the CarouselRenderInterface should extend this
 * class and must be stored under the Plugin/RenderMode directory.
 */
abstract class RenderModeBase extends PluginBase implements CarouselRenderInterface {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a RenderModeBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->moduleHandler = Drupal::moduleHandler();
  }

  /**
   * Get the mode id.
   *
   * @return string
   *   Mode id.
   */
  public function getId() {
    return $this->pluginDefinition['id'];
  }

  /**
   * Get the mode label.
   *
   * @return string
   *   Mode label.
   */
  public function getLabel() {
    return $this->pluginDefinition['label'];
  }

  /**
   * @inheritDoc
   */
  public function getUniqueId(CarouselInterface $carousel, array $configuration): string {
    return $carousel->id() . '_' . $configuration['last_updated'];
  }

  /**
   * @inheritDoc
   */
  public function getAllValues(CarouselInterface $carousel, array $configuration): array {

    $values = [
      'id' => $this->getUniqueId($carousel, $configuration),
      'status' => $carousel->get('status')->value == 1,
      'cid' => $carousel->id(),
      'label' => $carousel->label(),
      'config' => $configuration,
      'items' => [],
    ];

    foreach($carousel->getItems() as $item) {
      $values['items'][] = [
        'id' => $item->id(),
        'label' => $item->label(),
        'status' => $item->getStatus(),
        'media' => $item->getMedia(),
        'base64_image' => $item->getBase64Image(),
        'external_image' => $item->getExternalImage(),
        'background_color' => $this->opacityToHexValue($item->getBackgroundColor(), $item->getBackgroundOpacity()),
        'opacity' => $item->getBackgroundOpacity(),
        'title_color' => $item->getTitleColor(),
        'description_color' => $item->getDescriptionColor(),
        'text_alignment' => $item->getTextAlignment(),
        'text_position' => $item->getTextPosition(),
        'show_title' => $item->getShowTitle(),
        'title' => $item->getTitle(),
        'link' => $item->getLink(),
        'description' => $item->getDescription(),
      ];
    }

    // Clone all block configuration values.
    foreach ($configuration as $key => $value) {
      $values['config'][$key] = $value;
    }

    return $values;
  }

  /**
   * Convert opacity in the corresponding hex value.
   *
   * @param string $color_hex
   *   Color in hexadecimal format. (ej. "#FF5733").
   * @param float $opacity
   *   Opacity, float 0-1.
   *
   * @return string
   *   The color in RGBA hexadecimal (ej. "#FF573380").
   */
  public function opacityToHexValue(string $color_hex, float $opacity) {
    $color_hex = str_replace('#', '', $color_hex);
    $alpha = dechex((int) round($opacity * 255));
    // Ensure that alfa channel has two digits
    $alpha = str_pad($alpha, 2, '0', STR_PAD_LEFT);
    return '#' . $color_hex . $alpha;
  }

  /**
   * Get the module path.
   *
   * @return string
   *   The module path.
   */
  public function getModulePath(): string {
    return $this->moduleHandler->getModule('easy_carousel')->getPath();
  }
}
